@@ -1,26 +1,33 @@
 """The clients that interact with the service APIs."""
 
 import base64
+import binascii
 import hashlib
 import json
 import logging
 import re
 import time
 from abc import ABC, abstractmethod
-from typing import Generator, Sequence, Tuple, Union
+from pprint import pformat
+from typing import Any, Dict, Generator, Optional, Sequence, Tuple, Union
 
-import click
+import deezer
+from click import launch, secho
+from Cryptodome.Cipher import AES
 
 from .constants import (
     AGENT,
     AVAILABLE_QUALITY_IDS,
     DEEZER_BASE,
     DEEZER_DL,
+    DEEZER_FORMATS,
     DEEZER_MAX_Q,
     QOBUZ_BASE,
     QOBUZ_FEATURED_KEYS,
+    SOUNDCLOUD_APP_VERSION,
     SOUNDCLOUD_BASE,
     SOUNDCLOUD_CLIENT_ID,
+    SOUNDCLOUD_USER_ID,
     TIDAL_AUTH_URL,
     TIDAL_BASE,
     TIDAL_CLIENT_INFO,
@@ -35,7 +42,7 @@ from .exceptions import (
     MissingCredentials,
 )
 from .spoofbuz import Spoofer
-from .utils import gen_threadsafe_session, get_quality
+from .utils import gen_threadsafe_session, get_quality, safe_get
 
 logger = logging.getLogger("streamrip")
 
@@ -79,7 +86,7 @@ class Client(ABC):
         pass
 
     @abstractmethod
-    def get_file_url(self, track_id, quality=3) -> Union[dict, str]:
+    def get_file_url(self, track_id, quality=3) -> dict:
         """Get the direct download url dict for a file.
 
         :param track_id: id of the track
@@ -111,7 +118,7 @@ class QobuzClient(Client):
         :type pwd: str
         :param kwargs: app_id: str, secrets: list, return_secrets: bool
         """
-        click.secho(f"Logging into {self.source}", fg="green")
+        secho(f"Logging into {self.source}", fg="green")
         email: str = kwargs["email"]
         pwd: str = kwargs["pwd"]
         if not email or not pwd:
@@ -122,7 +129,7 @@ class QobuzClient(Client):
             return
 
         if (kwargs.get("app_id") or kwargs.get("secrets")) in (None, [], ""):
-            click.secho("Fetching tokens, this may take a few seconds.")
+            secho("Fetching tokens — this may take a few seconds.", fg="magenta")
             logger.info("Fetching tokens from Qobuz")
             spoofer = Spoofer()
             kwargs["app_id"] = spoofer.get_app_id()
@@ -268,12 +275,14 @@ class QobuzClient(Client):
         extras = {
             "artist": "albums",
             "playlist": "tracks",
-            "label": "albums",  # not tested
+            "label": "albums",
         }
 
         if media_type in extras:
             params.update({"extra": extras[media_type]})
 
+        logger.debug("request params: %s", params)
+
         epoint = f"{media_type}/get"
 
         response, status_code = self._api_request(epoint, params)
@@ -336,6 +345,7 @@ class QobuzClient(Client):
         if status_code == 401:
             raise AuthenticationError(f"Invalid credentials from params {params}")
         elif status_code == 400:
+            logger.debug(resp)
             raise InvalidAppIdError(f"Invalid app id from params {params}")
         else:
             logger.info("Logged in to Qobuz")
@@ -406,6 +416,7 @@ class QobuzClient(Client):
         logging.debug(f"Calling API with endpoint {epoint} params {params}")
         r = self.session.get(f"{QOBUZ_BASE}/{epoint}", params=params)
         try:
+            logger.debug(r.text)
             return r.json(), r.status_code
         except Exception:
             logger.error("Problem getting JSON. Status code: %s", r.status_code)
@@ -434,6 +445,166 @@ class DeezerClient(Client):
 
     def __init__(self):
         """Create a DeezerClient."""
+        self.client = deezer.Deezer(accept_language="en-US,en;q=0.5")
+        # self.session = gen_threadsafe_session()
+
+        # no login required
+        self.logged_in = False
+
+    def search(self, query: str, media_type: str = "album", limit: int = 200) -> dict:
+        """Search API for query.
+
+        :param query:
+        :type query: str
+        :param media_type:
+        :type media_type: str
+        :param limit:
+        :type limit: int
+        """
+        # TODO: use limit parameter
+        try:
+            search_function = getattr(self.client.api, f"search_{media_type}")
+        except AttributeError:
+            raise Exception
+
+        response = search_function(query, limit=limit)
+        return response
+
+    def login(self, **kwargs):
+        """Log into Deezer.
+
+        :param kwargs:
+        """
+        try:
+            arl = kwargs["arl"]
+        except KeyError:
+            raise MissingCredentials
+
+        success = self.client.login_via_arl(arl)
+        if not success:
+            raise AuthenticationError
+
+        self.logged_in = True
+
+    def get(self, meta_id: Union[str, int], media_type: str = "album"):
+        """Get metadata.
+
+        :param meta_id:
+        :type meta_id: Union[str, int]
+        :param type_:
+        :type type_: str
+        """
+        GET_FUNCTIONS = {
+            "track": self.client.api.get_track,
+            "album": self.client.api.get_album,
+            "playlist": self.client.api.get_playlist,
+            "artist": self.client.api.get_artist,
+        }
+
+        get_item = GET_FUNCTIONS[media_type]
+        item = get_item(meta_id)
+        if media_type in ("album", "playlist"):
+            tracks = getattr(self.client.api, f"get_{media_type}_tracks")(
+                meta_id, limit=-1
+            )
+            item["tracks"] = tracks["data"]
+            item["track_total"] = len(tracks["data"])
+        elif media_type == "artist":
+            albums = self.client.api.get_artist_albums(meta_id)
+            item["albums"] = albums["data"]
+
+        logger.debug(item)
+        return item
+
+    def get_file_url(self, meta_id: str, quality: int = 2) -> dict:
+        """Get downloadable url for a track.
+
+        :param meta_id: The track ID.
+        :type meta_id: Union[str, int]
+        :param quality:
+        :type quality: int
+        """
+        # TODO: optimize such that all of the ids are requested at once
+        dl_info: Dict[str, Any] = {"quality": quality}
+
+        track_info = self.client.gw.get_track(meta_id)
+        logger.debug("Track info: %s", pformat(track_info))
+
+        dl_info["fallback_id"] = safe_get(track_info, "FALLBACK", "SNG_ID")
+
+        format_info = get_quality(quality, "deezer")
+        assert isinstance(format_info, tuple)
+        format_no, format_str = format_info
+
+        dl_info["size_to_quality"] = {
+            int(track_info.get(f"FILESIZE_{format}")): self._quality_id_from_filetype(
+                format
+            )
+            for format in DEEZER_FORMATS
+        }
+
+        token = track_info["TRACK_TOKEN"]
+        url = self.client.get_track_url(token, format_str)
+
+        if url is None:
+            url = self._get_encrypted_file_url(
+                meta_id, track_info["MD5_ORIGIN"], track_info["MEDIA_VERSION"]
+            )
+
+        dl_info["url"] = url
+        logger.debug(pformat(dl_info))
+        return dl_info
+
+    def _get_encrypted_file_url(
+        self, meta_id: str, track_hash: str, media_version: str
+    ):
+        format_number = 1
+
+        url_bytes = b"\xa4".join(
+            (
+                track_hash.encode(),
+                str(format_number).encode(),
+                str(meta_id).encode(),
+                str(media_version).encode(),
+            )
+        )
+        url_hash = hashlib.md5(url_bytes).hexdigest()
+        info_bytes = bytearray(url_hash.encode())
+        info_bytes.extend(b"\xa4")
+        info_bytes.extend(url_bytes)
+        info_bytes.extend(b"\xa4")
+        # Pad the bytes so that len(info_bytes) % 16 == 0
+        padding_len = 16 - (len(info_bytes) % 16)
+        info_bytes.extend(b"." * padding_len)
+
+        logger.debug("Info bytes: %s", info_bytes)
+        path = self._gen_url_path(info_bytes)
+        logger.debug(path)
+        return f"https://e-cdns-proxy-{track_hash[0]}.dzcdn.net/mobile/1/{path}"
+
+    def _gen_url_path(self, data):
+        return binascii.hexlify(
+            AES.new("jo6aey6haid2Teih".encode(), AES.MODE_ECB).encrypt(data)
+        ).decode("utf-8")
+
+    @staticmethod
+    def _quality_id_from_filetype(filetype: str) -> Optional[int]:
+        return {
+            "MP3_128": 0,
+            "MP3_256": 0,
+            "MP3_320": 1,
+            "FLAC": 2,
+        }.get(filetype)
+
+
+class DeezloaderClient(Client):
+    """DeezloaderClient."""
+
+    source = "deezer"
+    max_quality = 1
+
+    def __init__(self):
+        """Create a DeezloaderClient."""
         self.session = gen_threadsafe_session()
 
         # no login required
@@ -560,7 +731,7 @@ class TidalClient(Client):
             self._login_new_user()
 
         self.logged_in = True
-        click.secho("Logged into Tidal", fg="green")
+        secho("Logged into Tidal", fg="green")
 
     def get(self, item_id, media_type):
         """Public method that internally calls _api_get.
@@ -636,7 +807,7 @@ class TidalClient(Client):
 
     # ------------ Utilities to login -------------
 
-    def _login_new_user(self, launch: bool = True):
+    def _login_new_user(self, launch_url: bool = True):
         """Create app url where the user can log in.
 
         :param launch: Launch the browser.
@@ -644,12 +815,12 @@ class TidalClient(Client):
         """
         login_link = f"https://{self._get_device_code()}"
 
-        click.secho(
+        secho(
             f"Go to {login_link} to log into Tidal within 5 minutes.",
             fg="blue",
         )
-        if launch:
-            click.launch(login_link)
+        if launch_url:
+            launch(login_link)
 
         start = time.time()
         elapsed = 0.0
@@ -660,15 +831,14 @@ class TidalClient(Client):
                 # pending
                 time.sleep(4)
                 continue
-            elif status == 1:
-                # error checking
-                raise Exception
             elif status == 0:
                 # successful
                 break
             else:
                 raise Exception
 
+        self._update_authorization()
+
     def _get_device_code(self):
         """Get the device code that will be used to log in on the browser."""
         data = {
@@ -836,8 +1006,9 @@ class TidalClient(Client):
 
         params["countryCode"] = self.country_code
         params["limit"] = 100
-        r = self.session.get(f"{TIDAL_BASE}/{path}", params=params).json()
-        return r
+        r = self.session.get(f"{TIDAL_BASE}/{path}", params=params)
+        # r.raise_for_status()
+        return r.json()
 
     def _get_video_stream_url(self, video_id: str) -> str:
         """Get the HLS video stream url.
@@ -885,7 +1056,18 @@ class SoundCloudClient(Client):
 
     def __init__(self):
         """Create a SoundCloudClient."""
-        self.session = gen_threadsafe_session(headers={"User-Agent": AGENT})
+        self.session = gen_threadsafe_session(
+            headers={
+                "User-Agent": AGENT,
+                "Host": "api-v2.soundcloud.com",
+                "Origin": "https://soundcloud.com",
+                "Referer": "https://soundcloud.com/",
+                "Sec-Fetch-Dest": "empty",
+                "Sec-Fetch-Mode": "cors",
+                "Sec-Fetch-Site": "same-site",
+                "Sec-GPC": "1",
+            }
+        )
 
     def login(self):
         """Login is not necessary for SoundCloud."""
@@ -946,7 +1128,7 @@ class SoundCloudClient(Client):
             resp, _ = self._get(url, no_base=True)
             return {"url": resp["url"], "type": "mp3"}
 
-    def search(self, query: str, media_type="album"):
+    def search(self, query: str, media_type="album", limit=50, offset=50):
         """Search for a query.
 
         :param query:
@@ -954,7 +1136,14 @@ class SoundCloudClient(Client):
         :param media_type: Can be album, though it will return a playlist
         response.
         """
-        params = {"q": query}
+        params = {
+            "q": query,
+            "facet": "genre",
+            "user_id": SOUNDCLOUD_USER_ID,
+            "limit": limit,
+            "offset": offset,
+            "linked_partitioning": "1",
+        }
         resp, _ = self._get(f"search/{media_type}s", params=params)
         return resp
 
@@ -968,16 +1157,24 @@ class SoundCloudClient(Client):
         :param resp_obj: Return the object returned by `requests.get` instead
         of the json response dict.
         """
-        if params is None:
-            params = {}
-        params["client_id"] = SOUNDCLOUD_CLIENT_ID
+        param_arg = params
+        params = {
+            "client_id": SOUNDCLOUD_CLIENT_ID,
+            "app_version": SOUNDCLOUD_APP_VERSION,
+            "app_locale": "en",
+        }
+        if param_arg is not None:
+            params.update(param_arg)
+
         if no_base:
             url = path
         else:
             url = f"{SOUNDCLOUD_BASE}/{path}"
 
-        logger.debug(f"Fetching url {url}")
+        logger.debug("Fetching url %s", url)
+        logger.debug("Parameters: %s", params)
         r = self.session.get(url, params=params)
+        logger.debug(r.text)
         if resp_obj:
             return r
 
