@@ -1,25 +1,13 @@
 """Constants that are kept in one place."""
 
-import os
-from pathlib import Path
-
-import click
 import mutagen.id3 as id3
 
-APPNAME = "streamrip"
-
-CACHE_DIR = click.get_app_dir(APPNAME)
-CONFIG_DIR = click.get_app_dir(APPNAME)
-CONFIG_PATH = os.path.join(CONFIG_DIR, "config.toml")
-LOG_DIR = click.get_app_dir(APPNAME)
-DB_PATH = os.path.join(LOG_DIR, "downloads.db")
-
-HOME = Path.home()
-DOWNLOADS_DIR = os.path.join(HOME, "StreamripDownloads")
-
 AGENT = "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:83.0) Gecko/20100101 Firefox/83.0"
 
 TIDAL_COVER_URL = "https://resources.tidal.com/images/{uuid}/{width}x{height}.jpg"
+SOUNDCLOUD_CLIENT_ID = "QFciLWLC1GS4P3EZvXIjA3jKhKO5pKB3"
+SOUNDCLOUD_USER_ID = "672320-86895-162383-801513"
+SOUNDCLOUD_APP_VERSION = "1626941202"
 
 
 QUALITY_DESC = {
@@ -130,7 +118,14 @@ PHON_COPYRIGHT = "\u00a9"
 FLAC_MAX_BLOCKSIZE = 16777215  # 16.7 MB
 
 # TODO: give these more descriptive names
-TRACK_KEYS = ("tracknumber", "artist", "albumartist", "composer", "title")
+TRACK_KEYS = (
+    "tracknumber",
+    "artist",
+    "albumartist",
+    "composer",
+    "title",
+    "albumcomposer",
+)
 ALBUM_KEYS = (
     "albumartist",
     "title",
@@ -138,7 +133,7 @@ ALBUM_KEYS = (
     "bit_depth",
     "sampling_rate",
     "container",
-    "composer",
+    "albumcomposer",
 )
 # TODO: rename these to DEFAULT_FOLDER_FORMAT etc
 FOLDER_FORMAT = (
@@ -147,20 +142,6 @@ FOLDER_FORMAT = (
 TRACK_FORMAT = "{tracknumber}. {artist} - {title}"
 
 
-# ------------------ Regexes ------------------- #
-URL_REGEX = (
-    r"https?://(?:www|open|play|listen)?\.?(qobuz|tidal|deezer)\.com(?:(?:/"
-    r"(album|artist|track|playlist|video|label))|(?:\/[-\w]+?))+\/([-\w]+)"
-)
-SOUNDCLOUD_URL_REGEX = r"https://soundcloud.com/[-\w:/]+"
-SOUNDCLOUD_CLIENT_ID = "a3e059563d7fd3372b49b37f00a00bcf"
-LASTFM_URL_REGEX = r"https://www.last.fm/user/\w+/playlists/\w+"
-QOBUZ_INTERPRETER_URL_REGEX = (
-    r"https?://www\.qobuz\.com/\w\w-\w\w/interpreter/[-\w]+/[-\w]+"
-)
-DEEZER_DYNAMIC_LINK_REGEX = r"https://deezer\.page\.link/\w+"
-YOUTUBE_URL_REGEX = r"https://www\.youtube\.com/watch\?v=[-\w]+"
-
 TIDAL_MAX_Q = 7
 
 TIDAL_Q_MAP = {
@@ -172,6 +153,14 @@ TIDAL_Q_MAP = {
 
 DEEZER_MAX_Q = 6
 AVAILABLE_QUALITY_IDS = (0, 1, 2, 3, 4)
+DEEZER_FORMATS = {
+    "AAC_64",
+    "MP3_64",
+    "MP3_128",
+    "MP3_256",
+    "MP3_320",
+    "FLAC",
+}
 # video only for tidal
 MEDIA_TYPES = {"track", "album", "artist", "label", "playlist", "video"}
 
