@@ -11,7 +11,7 @@ from .exceptions import ConversionError
 
 logger = logging.getLogger("streamrip")
 
-SAMPLING_RATES = (44100, 48000, 88200, 96000, 176400, 192000)
+SAMPLING_RATES = {44100, 48000, 88200, 96000, 176400, 192000}
 
 
 class Converter:
@@ -98,11 +98,13 @@ class Converter:
             "ffmpeg",
             "-i",
             self.filename,
-            "-loglevel",
-            "panic",
-            "-c:a",
-            self.codec_lib,
         ]
+
+        if logger.getEffectiveLevel() != logging.DEBUG:
+            command.extend(("-loglevel", "panic"))
+
+        command.extend(("-c:a", self.codec_lib))
+
         if self.show_progress:
             command.append("-stats")
 
@@ -128,7 +130,7 @@ class Converter:
                 if int(self.bit_depth) == 16:
                     command.extend(["-sample_fmt", "s16"])
                 elif int(self.bit_depth) in (24, 32):
-                    command.extend(["-sample_fmt", "s32"])
+                    command.extend(["-sample_fmt", "s32p"])
                 else:
                     raise ValueError("Bit depth must be 16, 24, or 32")
             elif self.bit_depth is not None:
@@ -137,6 +139,8 @@ class Converter:
         # automatically overwrite
         command.extend(["-y", self.tempfile])
 
+        logger.debug(command)
+
         return command
 
     def _is_command_valid(self):
