@@ -10,7 +10,6 @@ from typing import Generator, Hashable, Iterable, Optional, Union
 from .constants import (
     ALBUM_KEYS,
     COPYRIGHT,
-    COVER_SIZES,
     FLAC_KEY,
     MP3_KEY,
     MP4_KEY,
@@ -19,7 +18,7 @@ from .constants import (
     TRACK_KEYS,
 )
 from .exceptions import InvalidContainerError, InvalidSourceError
-from .utils import get_quality_id, safe_get, tidal_cover_url, get_cover_urls
+from .utils import get_cover_urls, get_quality_id, safe_get
 
 logger = logging.getLogger("streamrip")
 
@@ -68,6 +67,7 @@ class TrackMetadata:
         self.album: str
         self.albumartist: str
         self.composer: Optional[str] = None
+        self.albumcomposer: Optional[str] = None
         self.comment: Optional[str] = None
         self.description: Optional[str] = None
         self.purchase_date: Optional[str] = None
@@ -134,7 +134,7 @@ class TrackMetadata:
             self.date = resp.get("release_date_original") or resp.get("release_date")
             self.copyright = resp.get("copyright")
             self.albumartist = safe_get(resp, "artist", "name")
-            self.composer = safe_get(resp, "composer", "name")
+            self.albumcomposer = safe_get(resp, "composer", "name")
             self.label = resp.get("label")
             self.description = resp.get("description")
             self.disctotal = (
@@ -197,9 +197,9 @@ class TrackMetadata:
             # not embedded
             self.explicit = bool(resp.get("parental_warning"))
             self.quality = 2
-            self.bit_depth = 16
+            self.bit_depth = None
             self.cover_urls = get_cover_urls(resp, self.__source)
-            self.sampling_rate = 44100
+            self.sampling_rate = None
             self.streamable = True
 
         elif self.__source == "soundcloud":
@@ -263,9 +263,9 @@ class TrackMetadata:
         :param work:
         :type work: str
         """
-        if version is not None:
+        if version is not None and version not in self.title:
             self.title = f"{self.title} ({version})"
-        if work is not None:
+        if work is not None and work not in self.title:
             logger.debug("Work found: %s", work)
             self.title = f"{work}: {self.title}"
 
