@@ -3,34 +3,146 @@
 from __future__ import annotations
 
 import base64
+import functools
+import hashlib
 import logging
-import os
 import re
-from string import Formatter
-from typing import Dict, Hashable, Optional, Tuple, Union
 from collections import OrderedDict
+from string import Formatter
+from typing import Dict, Hashable, Iterator, Optional, Tuple, Union
 
-import click
 import requests
+from click import secho, style
+from Cryptodome.Cipher import Blowfish
 from pathvalidate import sanitize_filename
 from requests.packages import urllib3
 from tqdm import tqdm
 
-from .constants import AGENT, TIDAL_COVER_URL, COVER_SIZES
+from .constants import COVER_SIZES, TIDAL_COVER_URL
 from .exceptions import InvalidQuality, InvalidSourceError, NonStreamable
 
 urllib3.disable_warnings()
 logger = logging.getLogger("streamrip")
 
 
+class DownloadStream:
+    """An iterator over chunks of a stream.
+
+    Usage:
+
+        >>> stream = DownloadStream('https://google.com', None)
+        >>> with open('google.html', 'wb') as file:
+        >>>     for chunk in stream:
+        >>>         file.write(chunk)
+
+    """
+
+    is_encrypted = re.compile("/m(?:obile|edia)/")
+
+    def __init__(
+        self,
+        url: str,
+        source: str = None,
+        params: dict = None,
+        headers: dict = None,
+        item_id: str = None,
+    ):
+        """Create an iterable DownloadStream of a URL.
+
+        :param url: The url to download
+        :type url: str
+        :param source: Only applicable for Deezer
+        :type source: str
+        :param params: Parameters to pass in the request
+        :type params: dict
+        :param headers: Headers to pass in the request
+        :type headers: dict
+        :param item_id: (Only for Deezer) the ID of the track
+        :type item_id: str
+        """
+        self.source = source
+        self.session = gen_threadsafe_session(headers=headers)
+
+        self.id = item_id
+        if isinstance(self.id, int):
+            self.id = str(self.id)
+
+        if params is None:
+            params = {}
+
+        self.request = self.session.get(
+            url, allow_redirects=True, stream=True, params=params
+        )
+        self.file_size = int(self.request.headers.get("Content-Length", 0))
+
+        if self.file_size == 0 and not self.url.endswith(".jpg"):
+            raise NonStreamable("File not found.")
+
+    def __iter__(self) -> Iterator:
+        """Iterate through chunks of the stream.
+
+        :rtype: Iterator
+        """
+        if self.source == "deezer" and self.is_encrypted.search(self.url) is not None:
+            assert isinstance(self.id, str), self.id
+
+            blowfish_key = self._generate_blowfish_key(self.id)
+            return (
+                (self._decrypt_chunk(blowfish_key, chunk[:2048]) + chunk[2048:])
+                if len(chunk) >= 2048
+                else chunk
+                for chunk in self.request.iter_content(2048 * 3)
+            )
+
+        return self.request.iter_content(chunk_size=1024)
+
+    @property
+    def url(self):
+        """Return the requested url."""
+        return self.request.url
+
+    def __len__(self) -> int:
+        """Return the value of the "Content-Length" header.
+
+        :rtype: int
+        """
+        return self.file_size
+
+    @staticmethod
+    def _generate_blowfish_key(track_id: str):
+        """Generate the blowfish key for Deezer downloads.
+
+        :param track_id:
+        :type track_id: str
+        """
+        SECRET = "g4el58wc0zvf9na1"
+        md5_hash = hashlib.md5(track_id.encode()).hexdigest()
+        # good luck :)
+        return "".join(
+            chr(functools.reduce(lambda x, y: x ^ y, map(ord, t)))
+            for t in zip(md5_hash[:16], md5_hash[16:], SECRET)
+        ).encode()
+
+    @staticmethod
+    def _decrypt_chunk(key, data):
+        """Decrypt a chunk of a Deezer stream.
+
+        :param key:
+        :param data:
+        """
+        return Blowfish.new(
+            key, Blowfish.MODE_CBC, b"\x00\x01\x02\x03\x04\x05\x06\x07"
+        ).decrypt(data)
+
+
 def safe_get(d: dict, *keys: Hashable, default=None):
     """Traverse dict layers safely.
 
     Usage:
         >>> d = {'foo': {'bar': 'baz'}}
-        >>> _safe_get(d, 'baz')
+        >>> safe_get(d, 'baz')
         None
-        >>> _safe_get(d, 'foo', 'bar')
+        >>> safe_get(d, 'foo', 'bar')
         'baz'
 
     :param d:
@@ -50,7 +162,28 @@ def safe_get(d: dict, *keys: Hashable, default=None):
     return res
 
 
-def get_quality(quality_id: int, source: str) -> Union[str, int]:
+__QUALITY_MAP: Dict[str, Dict[int, Union[int, str, Tuple[int, str]]]] = {
+    "qobuz": {
+        1: 5,
+        2: 6,
+        3: 7,
+        4: 27,
+    },
+    "deezer": {
+        0: (9, "MP3_128"),
+        1: (3, "MP3_320"),
+        2: (1, "FLAC"),
+    },
+    "tidal": {
+        0: "LOW",  # AAC
+        1: "HIGH",  # AAC
+        2: "LOSSLESS",  # CD Quality
+        3: "HI_RES",  # MQA
+    },
+}
+
+
+def get_quality(quality_id: int, source: str) -> Union[str, int, Tuple[int, str]]:
     """Get the source-specific quality id.
 
     :param quality_id: the universal quality id (0, 1, 2, 4)
@@ -59,33 +192,7 @@ def get_quality(quality_id: int, source: str) -> Union[str, int]:
     :type source: str
     :rtype: Union[str, int]
     """
-    q_map: Dict[int, Union[int, str]]
-    if source == "qobuz":
-        q_map = {
-            1: 5,
-            2: 6,
-            3: 7,
-            4: 27,
-        }
-    elif source == "tidal":
-        q_map = {
-            0: "LOW",  # AAC
-            1: "HIGH",  # AAC
-            2: "LOSSLESS",  # CD Quality
-            3: "HI_RES",  # MQA
-        }
-    elif source == "deezer":
-        q_map = {
-            0: 128,
-            1: 320,
-            2: 1411,
-        }
-    else:
-        raise InvalidSourceError(source)
-
-    possible_keys = set(q_map.keys())
-    assert quality_id in possible_keys, f"{quality_id} must be in {possible_keys}"
-    return q_map[quality_id]
+    return __QUALITY_MAP[source][quality_id]
 
 
 def get_quality_id(bit_depth: Optional[int], sampling_rate: Optional[int]):
@@ -131,46 +238,6 @@ def get_stats_from_quality(
         raise InvalidQuality(quality_id)
 
 
-def tqdm_download(url: str, filepath: str, params: dict = None, desc: str = None):
-    """Download a file with a progress bar.
-
-    :param url: url to direct download
-    :param filepath: file to write
-    :type url: str
-    :type filepath: str
-    """
-    logger.debug(f"Downloading {url} to {filepath} with params {params}")
-    if params is None:
-        params = {}
-
-    session = gen_threadsafe_session()
-    r = session.get(url, allow_redirects=True, stream=True, params=params)
-    total = int(r.headers.get("content-length", 0))
-    logger.debug(f"File size = {total}")
-    if total < 1000 and not url.endswith("jpg") and not url.endswith("png"):
-        raise NonStreamable(url)
-
-    try:
-        with open(filepath, "wb") as file, tqdm(
-            total=total,
-            unit="iB",
-            unit_scale=True,
-            unit_divisor=1024,
-            desc=desc,
-            dynamic_ncols=True,
-            # leave=False,
-        ) as bar:
-            for data in r.iter_content(chunk_size=1024):
-                size = file.write(data)
-                bar.update(size)
-    except Exception:
-        try:
-            os.remove(filepath)
-        except OSError:
-            pass
-        raise
-
-
 def clean_format(formatter: str, format_info):
     """Format track or folder names sanitizing every formatter key.
 
@@ -232,14 +299,14 @@ def decrypt_mqa_file(in_path, out_path, encryption_key):
         from Crypto.Cipher import AES
         from Crypto.Util import Counter
     except (ImportError, ModuleNotFoundError):
-        click.secho(
+        secho(
             "To download this item in MQA, you need to run ",
             fg="yellow",
             nl=False,
         )
-        click.secho("pip3 install pycryptodome --upgrade", fg="blue", nl=False)
-        click.secho(".")
-        raise click.Abort
+        secho("pip3 install pycryptodome --upgrade", fg="blue", nl=False)
+        secho(".")
+        exit()
 
     # Do not change this
     master_key = "UIlTTEMmmLfGowo/UC60x2H45W6MdGgTRfo/umg4754="
@@ -318,53 +385,10 @@ def decho(message, fg=None):
     :param fg: ANSI color with which to display the message on the
     screen
     """
-    click.secho(message, fg=fg)
+    secho(message, fg=fg)
     logger.debug(message)
 
 
-interpreter_artist_regex = re.compile(r"getSimilarArtist\(\s*'(\w+)'")
-
-
-def extract_interpreter_url(url: str) -> str:
-    """Extract artist ID from a Qobuz interpreter url.
-
-    :param url: Urls of the form "https://www.qobuz.com/us-en/interpreter/{artist}/download-streaming-albums"
-    :type url: str
-    :rtype: str
-    """
-    session = gen_threadsafe_session({"User-Agent": AGENT})
-    r = session.get(url)
-    match = interpreter_artist_regex.search(r.text)
-    if match:
-        return match.group(1)
-
-    raise Exception(
-        "Unable to extract artist id from interpreter url. Use a "
-        "url that contains an artist id."
-    )
-
-
-deezer_id_link_regex = re.compile(
-    r"https://www\.deezer\.com/[a-z]{2}/(album|artist|playlist|track)/(\d+)"
-)
-
-
-def extract_deezer_dynamic_link(url: str) -> Tuple[str, str]:
-    """Extract a deezer url that includes an ID from a deezer.page.link url.
-
-    :param url:
-    :type url: str
-    :rtype: Tuple[str, str]
-    """
-    session = gen_threadsafe_session({"User-Agent": AGENT})
-    r = session.get(url)
-    match = deezer_id_link_regex.search(r.text)
-    if match:
-        return match.group(1), match.group(2)
-
-    raise Exception("Unable to extract Deezer dynamic link.")
-
-
 def get_container(quality: int, source: str) -> str:
     """Get the file container given the quality.
 
@@ -386,6 +410,14 @@ def get_container(quality: int, source: str) -> str:
 
 
 def get_cover_urls(resp: dict, source: str) -> dict:
+    """Parse a response dict containing cover info according to the source.
+
+    :param resp:
+    :type resp: dict
+    :param source:
+    :type source: str
+    :rtype: dict
+    """
     if source == "qobuz":
         cover_urls = OrderedDict(resp["image"])
         cover_urls["original"] = cover_urls["large"].replace("600", "org")
@@ -416,3 +448,78 @@ def get_cover_urls(resp: dict, source: str) -> dict:
         return cover_urls
 
     raise InvalidSourceError(source)
+
+
+def downsize_image(filepath: str, width: int, height: int):
+    """Downsize an image.
+
+    If either the width or the height is greater than the image's width or
+    height, that dimension will not be changed.
+
+    :param filepath:
+    :type filepath: str
+    :param width:
+    :type width: int
+    :param height:
+    :type height: int
+    :raises: ValueError
+    """
+    from PIL import Image
+
+    image = Image.open(filepath)
+
+    width = min(width, image.width)
+    height = min(height, image.height)
+
+    resized_image = image.resize((width, height))
+    resized_image.save(filepath)
+
+
+TQDM_THEMES = {
+    "plain": None,
+    "dainty": (
+        "{desc} |{bar}| "
+        + style("{remaining}", fg="magenta")
+        + " left at "
+        + style("{rate_fmt}{postfix} ", fg="cyan", bold=True)
+    ),
+}
+
+TQDM_DEFAULT_THEME = "dainty"
+
+TQDM_BAR_FORMAT = TQDM_THEMES["dainty"]
+
+
+def set_progress_bar_theme(theme: str):
+    """Set the theme of the tqdm progress bar.
+
+    :param theme:
+    :type theme: str
+    """
+    global TQDM_BAR_FORMAT
+    TQDM_BAR_FORMAT = TQDM_THEMES[theme]
+
+
+def tqdm_stream(
+    iterator: DownloadStream, desc: Optional[str] = None
+) -> Iterator[bytes]:
+    """Return a tqdm bar with presets appropriate for downloading large files.
+
+    :param iterator:
+    :type iterator: DownloadStream
+    :param desc: Description to add for the progress bar
+    :type desc: Optional[str]
+    :rtype: Iterator
+    """
+    with tqdm(
+        total=len(iterator),
+        unit="B",
+        unit_scale=True,
+        unit_divisor=1024,
+        desc=desc,
+        dynamic_ncols=True,
+        bar_format=TQDM_BAR_FORMAT,
+    ) as bar:
+        for chunk in iterator:
+            bar.update(len(chunk))
+            yield chunk
